/*---------------------------------------------------------------------------- 
 * $Id:$
 * Project  : The making of "MIB-tripper". 
 *		A MIB-browser implemented using the UT-SNMP package.
 * Filename : mibTree.h
 * Date     : october 1994
 * Purpose  : This file and its ".c"-file implement an interface to store,
 *		to maintain and to move around in a MIB-tree.
 * Author   : Erwin Bonsma
 *----------------------------------------------------------------------------
 */

#ifndef MIBTREE_H
#define MIBTREE_H 

#include <tripMisc.h>
#include <subId.h>

typedef struct mtNode {
  mtSubId subId;		/* Sub-identifier in OID, range 0..2^32-1. */				
  struct mtNode *father;
  struct mtNode *son;
  struct mtNode *brother;
  ObjectSyntax *value;
} mtNode;

typedef struct mtTree {
  mtNode *rootSub;
  mtNode *current;
  AsnOid *curOid;
} mtTree;  


/*----------------------------------------------------------------------------
 * Function   : mtNewTree
 * Arguments  :	mtTree		**tree		(out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function allocates memory for a new tree. The datafields are
 *		also initialised.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
mtNewTree(mtTree **tree);


/*----------------------------------------------------------------------------
 * Function   : mtAddObject
 * Arguments  : mtTree		*tree		(in/out)
 *		ObjectName	name		(in)
 *		ObjectSyntax	syntax		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function adds an object to the tree.
 * Remarks    : This function can also be used to update an object in the
 *		tree.
 *----------------------------------------------------------------------------
 */
STATUS 
mtAddObject(mtTree *tree, ObjectName name, ObjectSyntax syntax);


/*----------------------------------------------------------------------------
 * Function   :	mtShowTree
 * Arguments  :	mtTree		*tree		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function displays the total tree.
 * Remarks    :	Used for debugging only.
 *----------------------------------------------------------------------------
 */
STATUS
mtShowTree(mtTree *tree);


/*----------------------------------------------------------------------------
 * Function   :	mtKillTree
 * Arguments  :	mtTree		**tree		(in/out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function disposes all memory used by tree.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
mtKillTree(mtTree **tree);


/*----------------------------------------------------------------------------
 * Function   :	mtGoUp
 * Arguments  :	mtTree		*tree		(in/out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function changes current node to the node a level up in the
 *		MIB-tree.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
mtGoUp(mtTree *tree);


/*----------------------------------------------------------------------------
 * Function   :	mtGoDown
 * Arguments  :	mtTree		*tree		(in/out)
 *		mtSubId		subId		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function changes current node to the node a level deeper
 *		identified by the sub-identifier.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
mtGoDown(mtTree *tree, mtSubId subId);


/*----------------------------------------------------------------------------
 * Function   :	mtGoTo
 * Arguments  :	mtTree		*tree		(in/out)
 *		ObjectName	name		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function changes the current node to the node identified by
 *		the object-identifier.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
mtGoTo(mtTree *tree, ObjectName name);

#endif  /* MIBTREE_H */
