/*---------------------------------------------------------------------------- 
 * $Id:$
 * Project  : The making of "MIB-tripper". 
 *		A MIB-browser implemented using the UT-SNMP package.
 * Filename : tripMisc.c
 * Date     : october 1994
 * Purpose  : This file and its headerfile define functions to manipulate and
 *		to retreive subidentifiers within an objectidentifier.
 * Author   : Erwin Bonsma
 *----------------------------------------------------------------------------
 */
 
#include <subId.h>


/*----------------------------------------------------------------------------
 * Function   :	subIdGet
 * Arguments  :	ObjectName	*name		(in)
 *		int		nr		(in)
 *		mtSubid		*subId		(out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function returns the sub-identifier at the given position in
 *		the OID.
 * Remarks    :	This code is for the most part copied from the function
 *		ASCIItoOID (spmMisc.c)
 *----------------------------------------------------------------------------
 */ 
STATUS 
subIdGet(ObjectName *name, int nr, mtSubId *subId)
{ int i;		/* position in ObjectName-string */
  int arcNum;
  int firstArcNum;
  int secondArcNum;
  int arcNr;		/* number of subidentifiers in OID from the	*/
			/* begin of the string till the i-th position.	*/


  if ( (name == NULL) || (nr==0) )
    return FAILED;

  for (arcNum = 0, i=0; (i < name->octetLen) && (name->octs[i] & 0x80);i++)
    arcNum = (arcNum << 7) + (name->octs[i] & 0x7f);

  arcNum = (arcNum << 7) + (name->octs[i] & 0x7f);
  i++;
  
  firstArcNum = arcNum / 40;
  if (firstArcNum > 2 )
    firstArcNum = 2;
  arcNr=1;  
  
  secondArcNum = arcNum - (firstArcNum * 40); 
  arcNr=2;     
    
  for ( ; (i < name->octetLen) && (arcNr < nr); ) {
    for (arcNum = 0 ; (i < name->octetLen) && (name->octs[i] & 0x80);i++)
      arcNum = (arcNum << 7) + (name->octs[i] & 0x7f);

      arcNum = (arcNum << 7) + (name->octs[i] & 0x7f);
       
      arcNr++;
	  
      i++;
    }
    
  if (arcNr<nr) 
    return FAILED;
	
  if (nr==1) 
    *subId=firstArcNum;
  else if (nr==2)
    *subId=secondArcNum;
  else 
    *subId=arcNum;
    
  return OK;    
}


/*----------------------------------------------------------------------------
 * Function   :	subIdAppend
 * Arguments  :	ObjectName	*name		(in/out)
 *		mtSubId		subId		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function adds a subidentifier at the end of the OID.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS 
subIdAppend(ObjectName *name, mtSubId subId)
{ int shift;

  for (shift=4; (shift>0) && 
		( ((subId >> (shift*7)) & 0x7f )==0 ); shift--)
    /* void */ ;
     		
  for ( ; shift>=0; shift --)
    name->octs[ name->octetLen++ ] = ( (subId >> (shift*7)) & 0x7f ) +
                                     ( (shift>0) * 0x80 );    
  
  return OK;     		    
}


/*----------------------------------------------------------------------------
 * Function   :	subIdDelLast
 * Arguments  : ObjectName	*name		(in/out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function removes the last subidentifier from the OID.
 * Remarks    : Returns FAILED if oidsize<=1 byte. This to avoid problems
 *		with "1.3." encoded part of OID.
 *----------------------------------------------------------------------------
 */
STATUS 
subIdDelLast(ObjectName *name)
{ if ( name->octetLen <= 1)
    return FAILED;
    
  do {
    name->octetLen--;
  }
  while ((name->octetLen>0) &&		/* Not first byte in oid and 	    */
	( (name->octs[name->octetLen-1] & 0x80)>0 ));
					/* not last byte of previous subId. */
  return OK;  
}

