/*---------------------------------------------------------------------------- 
 * $Id:$
 * Project  : The making of "MIB-tripper". 
 *		A MIB-browser implemented using the UT-SNMP package.
 * Filename : tripMisc.c
 * Date     : october 1994
 * Purpose  : This file and its ".c"-file define functions to manipulate and
 *		to retreive subidentifiers within an objectidentifier.
 * Author   : Erwin Bonsma
 *----------------------------------------------------------------------------
 */
 
#ifndef SUBID_H
#define SUBID_H 

#include <snmpConst.h>
#include <snmpTypes.h>
#include <foid.h>

typedef int mtSubId; 


/*----------------------------------------------------------------------------
 * Function   :	subIdGet
 * Arguments  :	ObjectName	*name		(in)
 *		int		nr		(in)
 *		mtSubid		*subId		(out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function returns the sub-identifier at the given position in
 *		the OID.
 * Remarks    :	This code is for the most part copied from the function
 *		ASCIItoOID (spmMisc.c)
 *----------------------------------------------------------------------------
 */ 
STATUS
subIdGet(ObjectName *name, int nr, mtSubId *subId);


/*----------------------------------------------------------------------------
 * Function   :	subIdAppend
 * Arguments  :	ObjectName	*name		(in/out)
 *		mtSubId		subId		(in)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function adds a subidentifier at the end of the OID.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
subIdAppend(ObjectName *name, mtSubId subId);


/*----------------------------------------------------------------------------
 * Function   :	subIdDelLast
 * Arguments  : ObjectName	*name		(in/out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function removes the last subidentifier from the OID.
 * Remarks    : Returns FAILED if oidsize<=1 byte. This to avoid problems
 *		with "1.3." encoded part of OID.
 *----------------------------------------------------------------------------
 */
STATUS
subIdDelLast(ObjectName *name);

#endif /* SUBID_H */
