/*---------------------------------------------------------------------------- 
 * $Id:$
 * Project  : The making of "MIB-tripper". 
 *		A MIB-browser implemented using the UT-SNMP package.
 * Filename : tripper.c
 * Date     : october 1994
 * Purpose  : This program makes it possible to browse through a MIB. 
 * Author   : Erwin Bonsma
 *----------------------------------------------------------------------------
 */
 
#include <tripper.h>


/*----------------------------------------------------------------------------
 * Function   :	tripFriendlyASCIItoOID
 * Arguments  :	FoidT		*foidBase	(in)
 *		char		*oidASCII	(in)
 *		ObjectName	*oid		(out)
 * Returns    :	STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function converts a OID from "friendly character string" 
 *		format to the encoded "ObjectName" format.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
tripFriendlyASCIItoOID(FoidT *foidBase, char *oidASCII, ObjectName *oid)
{ AsnOcts	octFriendly, octOID;
  STATUS	status;
  AsnInt	errIdx, desIdx;
  
  octFriendly.octs = malloc(STRINGBUFSIZE);
  octFriendly.octetLen = STRINGBUFSIZE;
  octOID.octs = malloc(OIDBUFSIZE);
  octOID.octetLen = OIDBUFSIZE;
  
  /* Try to convert "friendly" OID-string to plain OID-string */
  status=foidConv(foidBase,oidASCII,octFriendly,octOID,&desIdx,&errIdx);
  if ((status!=OK) && (errIdx!=NTOTAL))  
    strcpy(octOID.octs,oidASCII);	/* Friendly conversion went wrong, */
       					/* so use original OID-string for  */
       					/* the next conversion.            */
    
  /* Convert OID-string to OID */
  status=ASCII2OID(octOID.octs, oid);
  
  free( octOID.octs ); 
  free( octFriendly.octs ); 
  
  return status;
}


/*----------------------------------------------------------------------------
 * Function   :	tripGetFriendlySubId
 * Arguments  :	FoidT      *foidBase (in)
 *              ObjectName *oid      (in)
 *		char       *s        (out)               
 * Returns    : STATUS, OK on success, FAILED otherwise.
 * Purpose    :	Function returns the last subidentifier of the given OID in 
 *		friendlyname format.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
tripGetFriendlySubId(FoidT *foidBase, ObjectName *oid, char *s)
{ int		i, start, end;
  AsnInt	errIdx,desIdx;
  AsnOcts	octFriendly, octOID;
  char		oidASCII[STRINGBUFSIZE];
  STATUS	status;
     
  octFriendly.octs = malloc(OIDBUFSIZE);
  octFriendly.octetLen = OIDBUFSIZE;
  octOID.octs = malloc(OIDBUFSIZE);
  octOID.octetLen = OIDBUFSIZE;
  
  OIDtoASCII(oid,oidASCII);
  if (foidConv(foidBase,oidASCII,octFriendly,octOID,&desIdx,&errIdx)==OK) {  
       
    for (i=end=strlen(octFriendly.octs); (i>0),(octFriendly.octs[i]!='.'); i--)
      /* void */ ;
    
    if (i>0)        
      start=i+1;		/* First char subId is after point.	*/
    else
      start=0; 			/* No point found, OID is subId.	*/
         
    for (i=start; (i<=end); i++)
      s[i-start]=octFriendly.octs[i];
      
    status=OK;  
  }
  else {
    s[0]='\0';
    status=FAILED;
  }  
    
  free( octFriendly.octs );
  free( octOID.octs );
  
  return status;
}


/*----------------------------------------------------------------------------
 * Function   :	tripError
 * Arguments  :	char		*msg		(in)
 * Returns    : 
 * Purpose    :	Function sends error message to stderr-device. 
 *		friendlyname format.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void
tripError(char *msg)
{
  fprintf(stderr,"Error : %s\n",msg);
}
 

/*----------------------------------------------------------------------------
 * Function   :	tripReadMIB
 * Arguments  :	FoidT	*foidBase	(in)
 *		char	*targetHostname	(in)
 *		INTEGER *reqId		(in/out)
 *		mtTree	*tree		(out)
 * Returns    : STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function reads a (part of a) MIB using the GetBulk-command.
 *		User input specifies the boundaries of the part of the MIB
 *		being read.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
tripReadMIB(FoidT *foidBase,char *targetHostname,INTEGER *reqId,mtTree *tree)
{ char		oidASCII[STRINGBUFSIZE];
  INTEGER	cfReqId;
  QOS		qos=0;
  int		maxRepetitions=25,nonRepeaters=0;
  INTEGER	ErrTp, ErrIdx;
  VarBindList	*vbList;
  VarBind	*vb;
  VarBindChoice	*vbChoice;
  BOOLEAN	readTotalMIB=FALSE, passedUpperLimit=FALSE;
  ObjectName	*oid, *oidHigh, *oidLow;
  SPTYPE	type;
  STATUS	status=OK;

  oidLow = pNewOid(OIDBUFSIZE);
  oidHigh= pNewOid(OIDBUFSIZE);
  oid    = pNewOid(OIDBUFSIZE);
  
  do {
    printf("Give lower OID > ");
    gets(oidASCII);
  }
  while (tripFriendlyASCIItoOID(foidBase,oidASCII,oidLow)!=OK);
  
  do {
    printf("Give upper OID > ");
    gets(oidASCII);
  }
  while (tripFriendlyASCIItoOID(foidBase,oidASCII,oidHigh)!=OK);
 
  tmCopyOcts(oid,oidLow);
    
  while ((!readTotalMIB) && (!passedUpperLimit)) {
    vbList = mVlNew();  
    vb=mVbPut(mVbNew(255),oid,mVcPut(mVcNew(),VARBINDCHOICE_UNSPECIFIED,0) );
    
    mVlPut( vbList, vb);       

    OIDtoASCII(oid,oidASCII);
    printf("Send GetBulkRequest (Id=%i, oid=%s).\n",*reqId,oidASCII);
    snmpGetBulkRequest( targetHostname, NULL, *reqId, qos, 
                        maxRepetitions, nonRepeaters, vbList );
    /* pFreeVbl(vbList); --- Bug makes freeing impossible! --- */
			
    printf("Awaiting confirm.\n");
    while (snmpLook( &type )!=OK)	
      /* void */ ;
      
    vbList = mVlNew();  
    if (type!=GETBULK_CNF) {
      tripError("Unexpected confirm type.");
      status=FAILED;
    }  
    else {
      snmpGetBulkConfirm( &cfReqId, &ErrTp, &ErrIdx, &vbList );
      printf("Received GetBulkConfirm (Id=%i, ErrTp=%i, ErrIdx=%i).\n",
                cfReqId,ErrTp,ErrIdx);
                
      FOR_EACH_LIST_ELMT(vb, vbList) {
        vbChoice=vb->varBindChoice;
        
        switch (vbChoice->choiceId) {
          case VARBINDCHOICE_VALUE :           
            tmCopyOcts(oid,&(vb->name));/* Keep OID of last retreived	*/
            				/* variable binding.		*/
            passedUpperLimit = (CMPOID(oid,oidHigh)==GREATER);
            
            if (!passedUpperLimit) {
              OIDtoASCII(oid,oidASCII);
              printf("Add %s\n",oidASCII);           
              mtAddObject(tree,*oid,*(vbChoice->a.value));
            }            
            break;
          case VARBINDCHOICE_ENDOFMIBVIEW :
            readTotalMIB=TRUE; 
            break;
          default : 
            tripError("Unexpected VarBindChoice.");
            status=FAILED; 
            break;  
        }    
      }                   
    }
       
    /* pFreeVbl(vbList); ---Bug makes freeing impossible! --- */
	
    (*reqId)++;    
  }  
  
  free(oid);
  
  return status;	
}


/*----------------------------------------------------------------------------
 * Function   : tripInit	
 * Arguments  :	char		**arge		(in)
 *              INTEGER		*reqId		(in/out)
 *		char		*targetHostname	(out)
 *              mtTree		**tree		(out)
 *		FoidT		**foidBase	(out)
 * Returns    : 
 * Purpose    : Function takes all necessary initialisation steps.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void 
tripInit(char **arge, INTEGER *reqId, char *targetHostname, 
		mtTree **tree, FoidT **foidBase)
{ char		locAddr[STRINGBUFSIZE];  

  printf("%s, %s\n\n",PROGRAMNAME,PROGRAMVERSION);
  
  /* Construct an unique local address */		
  sprintf(locAddr,"MIB-tripper %i",getpid());
    
  printf("Opening connection with spm [local address = %s].\n",locAddr);
  if (snmpOpen( locAddr, arge )!=OK)
    tripError("snmpOpen failed.");
  else  
    printf("Ok\n");
    
  printf("Opening friendly-name database.\n");  
  if (foidOpen(foidBase)!=OK)
    tripError("foidOpen failed.");
  else
    printf("Ok\n");  
  
  mtNewTree(tree);
    
  printf("Give target hostname > ");
  gets(targetHostname);  
  
  tripReadMIB(*foidBase,targetHostname,reqId,*tree);
    
  printf("Goto 1.3, status %i\n",mtGoTo(*tree,org));  
}


/*----------------------------------------------------------------------------
 * Function   :	tripShowTree
 * Arguments  :	mtTree		*tree		(in)
 *		FoidT		*foidBase	(in)
 * Returns    : 
 * Purpose    : Function shows current position in tree. Current node and all
 *		child nodes are shown. Also extra info such as friendlyname
 *		description and value is displayed when availiable. 
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void 
tripShowTree(mtTree *tree, FoidT *foidBase)
{ char		oidASCII[STRINGBUFSIZE];
  char		*line;
  mtNode	*p;
  AsnInt	desIdx, errIdx;
  STATUS	status;
  AsnOcts	octFriendly, octOID, octDescr;
  ObjectName	*oid;
  
  oid = pNewOid(OIDBUFSIZE);
  
  octFriendly.octs = malloc(STRINGBUFSIZE);
  octFriendly.octetLen = STRINGBUFSIZE;
  octOID.octs = malloc(OIDBUFSIZE);
  octOID.octetLen = OIDBUFSIZE;
  octDescr.octs = malloc(DESCRIPTIONBUFSIZE);
  octDescr.octetLen = DESCRIPTIONBUFSIZE;
  
  printf("\n");
  printf("===========================================\n");
  printf("  %s : Take a Trip Through a MIB  \n",PROGRAMNAME);
  printf("===========================================\n\n");
  
  printf("Current node:\n");

  OIDtoASCII(tree->curOid,oidASCII);
  printf("\tOID:\n");
  printf("\t\t%s\n",oidASCII);
  
  status=foidConv(foidBase,oidASCII,octFriendly,octOID,&desIdx,&errIdx);
  if ((status==OK) || (errIdx==NTOTAL)) {
    printf("\tFriendly name:\n");    
    printf("\t\t%s\n",octFriendly.octs);
    if (desIdx>0) {
      foidDescription( desIdx, octDescr );       
      
      printf("\tDescription:\n");
      line = (char *) strtok( (char *)octDescr.octs, (char *)"\n" );
      while (line!=NULL) {
        printf("\t\t%s\n",line);
        line = (char *) strtok( (char *)NULL, (char *)"\n" );
      }
    }
  }  
  
  if (tree->current->value!=NULL) { 
    printf("\tValue:\n");
    printf("\t\t");
    PrintObjectSyntax(stdout, tree->current->value, 0);
    printf("\n");
  }    
 
  if ((p=tree->current->son)!=NULL) {
    printf("\nChild nodes:\n");
  
    oid->octetLen = tree->curOid->octetLen;
    memcpy(oid->octs,tree->curOid->octs,oid->octetLen);
     
    while (p!=NULL) { 
      printf("\t%i",p->subId);
  
      subIdAppend(oid,p->subId);
      tripGetFriendlySubId(foidBase,oid,oidASCII);      
      printf("\t%s\n",oidASCII);    
      subIdDelLast(oid);    
  
      p=p->brother;
    }
  }
    
  printf("\n");
      
  free( octFriendly.octs );
  free( octOID.octs );
  free( octDescr.octs );
  
  free(oid);
}


/*----------------------------------------------------------------------------
 * Function   :	tripGetChoice
 * Arguments  :	tripChoice	*choice		(out)
 *		int		*downVal	(out)              
 * Returns    : 
 * Purpose    : Function waits for correct user input and returns the choice
 *		the user made.
 * Remarks    : The variable downVal is only relevant when the user's choice
 *		is chDown. If the user entered a number a the prompt this
 *		number is interpreted as "Go down to node <number>".
 *----------------------------------------------------------------------------
 */
void 
tripGetChoice(tripChoice *choice, int *downVal)
{ int		l,i,key;
  char		s[STRINGBUFSIZE];
    
  printf("[G]oto\t[U]p\t[D]own\t[Q]uit\t[V]alueUpdate\n\n");  
  
  do {
    printf("Enter your choice >");
    gets(s);
    switch (s[0]) {
      case 'g' : ;
      case 'G' : *choice=chGoto; break;
      case 'u' : ;
      case 'U' : *choice=chUp; break;
      case 'd' : ;
      case 'D' : *downVal=NOIMMEDIATEDOWN;
                 *choice=chDown; break;
      case 'q' : ;
      case 'Q' : *choice=chQuit; break;
      case 'v' : ;
      case 'V' : *choice=chUpdateValue; break;      
      default  : l=strlen(s);
                 for (i=0; i<l, (isdigit(s[i])); i++)
      		   /* viod */ ;
      		   
                 if (i==l) {
                   *downVal=atoi(s);
                   *choice=chDown;
                 }  
                 else
                   *choice=chNone;       
    }        
  }  
  while (*choice==chNone);   
}


/*----------------------------------------------------------------------------
 * Function   :	tripGoUp
 * Arguments  :	mtTree		*tree		(in/out)              
 * Returns    : 
 * Purpose    : Function moves current node a level up in the MIB-tree.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void
tripGoUp(mtTree *tree)
{ 
  printf("\nAction : Up, status : %i\n",mtGoUp(tree));
}


/*----------------------------------------------------------------------------
 * Function   :	tripGoDown	
 * Arguments  :	mtTree		*tree		(in/out)
 *		int		downVal		(in)              
 * Returns    : 
 * Purpose    : Function moves current node a level deeper in the MIB-Tree. 
 * Remarks    : There are three ways to chose the node to go down to :
 *		1) If downVal<>NOIMMEDIATEDOWN go down to that node.
 *		2) Else if the current node has only one child go down to
 *			that node.
 *		3) Else ask the user the number of the node to go down to.
 *----------------------------------------------------------------------------
 */
void 
tripGoDown(mtTree *tree, int downVal)
{ char		s[STRINGBUFSIZE];
  mtSubId	subId;
  int		aSon=0;
  mtNode	*p;
  
  if (downVal==NOIMMEDIATEDOWN) { 
    p=tree->current->son;
    while ((p!=NULL) && (aSon<=1)) { 
      aSon++;
      p=p->brother;
    }  
  
    if (aSon!=0) 
      if (aSon==1)
        subId=tree->current->son->subId;
      else { 
        printf("Go down to >");
        gets(s);
        subId=atoi(s);
      }    
    else
      subId=0;
  }    
  else
    subId=downVal;
    
  printf("\nAction : Down, status : %i\n",mtGoDown(tree,subId));   
}


/*----------------------------------------------------------------------------
 * Function   : tripGoTo
 * Arguments  :	mtTree		*tree		(in/out)              
 * Returns    : 
 * Purpose    : Function moves current node to a node specified by the user.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void
tripGoTo(mtTree *tree, FoidT *foidBase)
{ char		s[STRINGBUFSIZE];
  ObjectName	*oid;
 
  oid=pNewOid(OIDBUFSIZE);  
  
  printf("Goto >");
  gets(s);

  if (tripFriendlyASCIItoOID(foidBase, s, oid) != OK)
    printf("Oid not correct.\n");  
  else  					
    printf("\nAction : Goto, status : %i\n",mtGoTo(tree,*oid));
           
  free(oid);        
}


/*----------------------------------------------------------------------------
 * Function   :	tripUpdateValue	
 * Arguments  :	mtTree		*tree		(in/out)
 *		char		*targetHostname	(in)
 *		INTEGER		*reqId		(in/out)              
 * Returns    : STATUS, OK on success, FAILED otherwise.
 * Purpose    : Function updates the value of the current node using the 
 *		Get-command.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
STATUS
tripUpdateValue(mtTree *tree, char *targetHostname, INTEGER *reqId)
{ VarBindList	*vbList;
  VarBind	*vb;
  VarBindChoice	*vbChoice;
  QOS		qos=0;
  INTEGER	ErrTp, ErrIdx, cfReqId;  
  SPTYPE	type;
  ObjectName	*oid;
  char		oidASCII[STRINGBUFSIZE];
  STATUS	status=OK;

  if (tree->current->value==NULL)
    return FAILED;
    
  oid = pNewOid(OIDBUFSIZE);  
  
  tmCopyOcts(oid,tree->curOid);
    
  vbList = mVlNew();  
  vb=mVbPut(mVbNew(255),oid,mVcPut(mVcNew(),VARBINDCHOICE_UNSPECIFIED,0) );
  mVlPut( vbList, vb);
   
  OIDtoASCII(oid,oidASCII);
  printf("\n");
  printf("Send GetRequest (Id=%i, oid=%s).\n",*reqId,oidASCII);
  snmpGetRequest( targetHostname, NULL, *reqId, qos, vbList );
  /* pFreeVbl(vbList); --- Bug makes freeing impossible! --- */
			
  printf("Awaiting confirm.\n");
  while (snmpLook( &type )!=OK)	
    /* void */ ;
     
  vbList = mVlNew();  
  if (type!=GET_CNF) {
    tripError("Unexpected confirm type.");
    status=FAILED;
  }  
  else { 
    snmpGetConfirm( &cfReqId, &ErrTp, &ErrIdx, &vbList );
    printf("Received GetConfirm (Id=%i, ErrTp=%i, ErrIdx=%i).\n",
                cfReqId,ErrTp,ErrIdx);
                
    vb=FIRST_LIST_ELMT(vbList);
    vbChoice=vb->varBindChoice;
    
    switch (vbChoice->choiceId) { 
      case VARBINDCHOICE_VALUE :
        OIDtoASCII(oid,oidASCII);
	printf("Update %s\n",oidASCII);
        mtAddObject(tree,*oid,*(vbChoice->a.value)); 
        break;
      default :
        tripError("Unexpected VarBindChoice.\n");
        status=FAILED; 
        break;  
    }   
                
  }  
  
  /* pFreeVbl(vbList); --- Bug makes freeing impossible! --- */  
  
  (*reqId)++;
       
  free(oid);
  
  return status;    
}
      

/*----------------------------------------------------------------------------
 * Function   :	tripDone
 * Arguments  :	mtTree		*tree		(in/out)
 *		FoidT		**foidBase	(in/out)              
 * Returns    : 
 * Purpose    : Function takes all steps necessary to terminate the program
 *		correct.
 * Remarks    :
 *----------------------------------------------------------------------------
 */
void
tripDone(mtTree **tree, FoidT **foidBase)
{ mtKillTree(tree);

  foidClose(foidBase);
  
  snmpClose( GRACEFULL );
}


void main(int argc, char **argv, char **arge)
{ mtTree 	*tree;
  tripChoice 	choice;
  FoidT		*foidBase;
  char		targetHostname[STRINGBUFSIZE];
  INTEGER	reqId=1000;
  int		downVal;
  
  tripInit(arge,&reqId,targetHostname,&tree,&foidBase);
  
  do { 
    tripShowTree(tree,foidBase);
  
    tripGetChoice(&choice,&downVal);    
    switch (choice) { 
      case chUp : 
    	tripGoUp(tree); 
    	break;
      case chDown :      
        tripGoDown(tree,downVal);
        break;
      case chGoto :
        tripGoTo(tree,foidBase);  	
        break;
      case chUpdateValue :
        tripUpdateValue(tree,targetHostname,&reqId);
        break;  
    }    
        
  } while (choice!=chQuit);
  
  tripDone(&tree,&foidBase);
}
